using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace MWDataGrabber
{
	public enum RecordType
	{
		Cell,
		Cont,
		Unknown
	}

	public class Record
    {
        protected string _id;
        protected string _name;
        protected int _length;
        protected int _header1;
        protected int _flags;
        private BinaryReader _source;
        protected RecordType _type;
        private string _namespace;
		private List<string> _subrecords = new List<string>();

        protected Record( BinaryReader source )
        {
            _source = source;
            _length = source.ReadInt32();
            _header1 = source.ReadInt32();
            _flags = source.ReadInt32();
        }

        public static Record Create( BinaryReader source )
        {
            string type = ReadString( source, 4 );
            Record newOne;
            switch ( type )
            {
				case "CELL":
                    newOne = new Records.Cell( source );
                    break;
				case "CONT":
					newOne = new Records.Cont( source );
					break;
                default:
                    newOne = new Records.Default( source );
                    break;
            }
            newOne.LoadData();
            return newOne;
        }

        private static string ReadString( BinaryReader source, int length )
        {
            if ( length == 0 )
            {
                return string.Empty;
            }
            else
            {
                byte[] buffer = new byte[length];
                source.Read( buffer, 0, length );
                return Encoding.ASCII.GetString( buffer ).TrimEnd( '\0' );
            }
        }

        protected int ReadInt()
        {
            return _source.ReadInt32();
        }

		protected short ReadInt16()
		{
			return _source.ReadInt16();
		}

        protected bool ReadBool()
        {
            return _source.ReadBoolean();
        }

		protected byte ReadByte()
		{
			return _source.ReadByte();
		}

        protected float ReadFloat()
        {
            return _source.ReadSingle();
        }

        protected string ReadString( int length )
        {
            return ReadString( _source, length );
        }

        protected void Seek( int offset )
        {
            _source.BaseStream.Seek( offset, SeekOrigin.Current );
        }

		protected byte[] ReadBinary( int length )
		{
			byte[] buf = new byte[length];
			_source.Read( buf, 0, length );
			return buf;
		}

        protected virtual void ProcessSubRecord( string id, int length )
        {
            switch(id)
            {
                case "NAME":
                    _id = ReadString( length );
                    break;
                case "FNAM":
                    _name = ReadString( length );
                    break;
                default:
					_subrecords.Add( id );
                    Seek( length );
                    break;
            }
        }

        private void LoadData()
        {
            int length = _length;
            while ( length > 0 )
            {
                string sub = ReadString( 4 );
                int sublen = ReadInt();
                length -= 8;
                ProcessSubRecord( sub, sublen );
                length -= sublen;
            }
        }

        public virtual string Id
        {
            get { return _id; }
        }

        public virtual string Name
        {
            get { return _name; }
        }

        public RecordType Type
        {
            get { return _type; }
        }

        public string Namespace
        {
            get { return _namespace; }
            set { _namespace = value; }
        }

		public List<string> SubRecords
		{
			get
			{
				return _subrecords;
			}
		}

        public override string ToString()
        {
            return _name;
        }

    }
}
